@@ -12,9 +12,11 @@ base vectors and inserted vectors, respectively.
 
 import collections
 import sys
+from typing import Dict, FrozenSet, Iterator, List, Tuple, Union
 
 import numpy as np
 
+from cr.cube.dimension import Dimension, _Element, _OrderSpec, _Subtotals
 from cr.cube.util import lazyproperty
 
 
@@ -25,31 +27,37 @@ class _BaseCollator:
     unweighted N (N = 0).
     """
 
-    def __init__(self, dimension, empty_idxs):
+    def __init__(self, dimension: Dimension, empty_idxs: Tuple[int]):
         self._dimension = dimension
         self._empty_idxs = tuple(empty_idxs) if empty_idxs else ()
 
     @lazyproperty
-    def _element_ids(self):
-        """Sequence of int element-id for each category or subvar in dimension.
+    def _elements(self) -> Tuple[_Element, ...]:
+        """Sequence of non-missing elements from dimension"""
+        return self._dimension.valid_elements
 
-        Element-ids appear in the order there were defined in the cube-result.
+    @lazyproperty
+    def _element_ids(self) -> Tuple[Union[str, int], ...]:
+        """Sequence of int or str element-id for each category or subvar in dimension.
+
+        Element-ids appear in the order they were defined in the cube-result. These
+        element ids include both true element ids and also derived element ids.
         """
         return self._dimension.element_ids
 
     @lazyproperty
-    def _hidden_idxs(self):
+    def _hidden_idxs(self) -> FrozenSet[int]:
         """frozenset of int element-idx of each vector for which to suppress display."""
         empty_idxs = self._empty_idxs if self._dimension.prune else ()
         return frozenset(empty_idxs + self._dimension.hidden_idxs)
 
     @lazyproperty
-    def _order_spec(self):
+    def _order_spec(self) -> _OrderSpec:
         """_OrderSpec object specifying ordering details."""
         return self._dimension.order_spec
 
     @lazyproperty
-    def _subtotals(self):
+    def _subtotals(self) -> _Subtotals:
         """Sequence of _Subtotal object for each inserted subtotal in dimension."""
         return self._dimension.subtotals
 
@@ -63,7 +71,7 @@ class _BaseAnchoredCollator(_BaseCollator):
     """
 
     @classmethod
-    def display_order(cls, dimension, empty_idxs):
+    def display_order(cls, dimension, empty_idxs) -> Tuple[int, ...]:
         """Return sequence of int element-idx specifying ordering of dimension elements.
 
         The returned indices are "signed", with positive indices applying to base
@@ -76,7 +84,7 @@ class _BaseAnchoredCollator(_BaseCollator):
         return cls(dimension, empty_idxs)._display_order
 
     @lazyproperty
-    def _display_order(self):
+    def _display_order(self) -> Tuple[int, ...]:
         """tuple of int element-idx for each element in assembly order.
 
         An assembled vector contains both base and inserted cells. The returned
@@ -86,31 +94,48 @@ class _BaseAnchoredCollator(_BaseCollator):
         hidden_idxs = self._hidden_idxs
         return tuple(
             idx
-            for _, idx in sorted(
-                self._base_element_orderings + self._insertion_orderings
+            for _, _, idx in sorted(
+                self._base_element_orderings
+                + self._insertion_orderings
+                + self._derived_element_orderings
             )
             if idx not in hidden_idxs
         )
 
     @lazyproperty
-    def _base_element_orderings(self):
-        """tuple of (int: position, int: idx) for each base-vector value.
+    def _base_element_orderings(self) -> Tuple[Tuple[int, int, int], ...]:
+        """tuple of (int: position, int: rel=0, int: idx) for each base-vector value.
 
-        The position of a base value is it's index in the ordered base vector.
+        The position of a base value is it's index in the ordered base vector. The
+        second item's value of 0 indicates that it's the base element and places it
+        between the insertions/derived values (which when anchored before the
+        base element are given a negative value, and when before, they get a positive
+        one).
         """
         return tuple(
-            (position, idx) for position, idx, _ in self._element_order_descriptors
+            (position, 0, idx) for position, idx, _ in self._element_order_descriptors
         )
 
     @lazyproperty
-    def _element_order_descriptors(self):
+    def _derived_element_orderings(self) -> Tuple[Tuple[int, int, int], ...]:
+        """Optional tuple of orderings for each derived-element value.
+
+        Is None for payload order, because zz9 places the derived elements in their
+        correct positions with no need for us to further sort them. However, when
+        an explicit order is set, this method must be overriden because we must
+        recalculate the derived element's positions.
+        """
+        return tuple()
+
+    @lazyproperty
+    def _element_order_descriptors(self) -> Tuple[int, int, int]:
         """tuple of (position, idx, element_id) triple for each element in dimension."""
         raise NotImplementedError(
             f"`{type(self).__name__}` must implement `._element_order_descriptors`"
         )
 
     @lazyproperty
-    def _element_positions_by_id(self):
+    def _element_positions_by_id(self) -> Dict[Union[int, str], int]:
         """dict of int base-element position keyed by that element's id.
 
         Allows O(1) lookup of base-element position by element-idx for purposes of
@@ -122,20 +147,20 @@ class _BaseAnchoredCollator(_BaseCollator):
         }
 
     @lazyproperty
-    def _insertion_orderings(self):
-        """tuple of (int: position, int: idx) for each inserted-vector value.
+    def _insertion_orderings(self) -> Tuple[Tuple[int, int, int], ...]:
+        """tuple of (int: position, int: rel, int: idx) for each inserted-vector value.
 
-        The position for an insertion is an int representation of its anchor and its
-        idx is the *negative* offset of its position in the opposing insertions
-        sequence (like -3, -2, -1 for a sequence of length 3). The negative idx
-        works just as well as the normal one for accessing the subtotal but insures
-        that an insertion at the same position as a base row always sorts *before*
-        the base row.
+        The first item ("position") refers to position of the insertion's anchor, and
+        the second ("rel") is an integer that can (theoretically) be either -1 and 1
+        that indicates whether it should be placed before or after its anchor (though at
+        the time of writing, all categorical insertions are after and so have a positive
+        1). The idx is the *negative* offset of its position in the opposing insertions
+        sequence (like -3, -2, -1 for a sequence of length 3). The negative idx works
+        just as well as the normal one for accessing the subtotal but insures that an
+        insertion at the same position as a base row always sorts *before* the base row.
 
-        The `position` int for a subtotal is 0 for anchor "top", sys.maxsize for
-        anchor "bottom", and int(anchor) + 1 for all others. The +1 ensures
-        a subtotal appears *after* the vector it is anchored to and can be interpreted
-        as the index of the base-element it should appear *before*.
+        The `position` int for a subtotal is -1 for anchor "top", sys.maxsize for anchor
+        "bottom", and int(anchor) for all others.
 
         Multiple insertions having the same anchor appear in payload order within that
         group. The strictly increasing insertion index values (-3 < -2 < -1) ensure
@@ -145,43 +170,39 @@ class _BaseAnchoredCollator(_BaseCollator):
         n_subtotals = len(subtotals)
         neg_idxs = tuple(i - n_subtotals for i in range(n_subtotals))
         return tuple(
-            (self._insertion_position(subtotal), neg_idx)
+            (*self._insertion_position(subtotal), neg_idx)
             for subtotal, neg_idx in zip(subtotals, neg_idxs)
         )
 
-    def _insertion_position(self, subtotal):
-        """Subtotal position expressed as int index among base-vector indices.
+    def _insertion_position(self, subtotal) -> Tuple[int, int]:
+        """Subtotal position expressed as tuple of ints index among base-vector indices.
 
-        The return value represents the payload-order base-vector idx *before which* the
-        subtotal should appear (even though subtotals appear *after* the row they are
-        anchored to.
+        The first item in the return value represents the payload-order base-vector idx
+        which the subtotal is "anchor"ed to. The second item can (theoretically) be
+        either a positive 1 or negative 1 to indicate whether it should be after or
+        before that element respectively. At the time of writing, subtotal insertions
+        can only be placed after their anchor (derived elements, aka MR insertions can
+        be placed before or after).
 
-        A subtotal with position `0` appears at the top, one with an anchor of `3`
-        appears *before* the base row at offset 3; `sys.maxsize` is used as the position
-        for a "bottom" anchored subtotal.
-
-        To make this work, the position of a subtotal is idx+1 of the base row it
-        is anchored to (for subtotals anchored to a row, not "top" or "bottom").
-        Combining this +1 characteristic with placing subtotals before rows with
-        idx=insertion_position produces the right positioning and also allows top and
-        bottom anchors to work while representing the position as a single non-negative
-        int.
+        A subtotal with position `(-1, 1)` appears at the top, one with an anchor of
+        `(3, 1)` appears *after* the base row at offset 3; `(sys.maxsize, 0)` is used as
+        the position for a "bottom" anchored subtotal.
         """
         anchor = subtotal.anchor
 
         # --- "top" and "bottom" have fixed position mappings ---
         if anchor == "top":
-            return 0
+            return (-1, 0)
         if anchor == "bottom":
-            return sys.maxsize
+            return (sys.maxsize, 0)
 
-        # --- otherwise look up anchor-element position by id and add 1, defaulting to
-        # --- bottom if target anchor element not found ---
+        # --- otherwise look up anchor-element position by id, defaulting to bottom if
+        # --- target anchor element not found ---
         element_id = int(anchor)
         return (
-            self._element_positions_by_id[element_id] + 1
+            (self._element_positions_by_id[element_id], 1)
             if element_id in self._element_positions_by_id
-            else sys.maxsize
+            else (sys.maxsize, 0)
         )
 
 
@@ -189,7 +210,61 @@ class ExplicitOrderCollator(_BaseAnchoredCollator):
     """Orders elements in the sequence specified in order transform."""
 
     @lazyproperty
-    def _element_order_descriptors(self):
+    def _derived_element_orderings(self) -> Tuple[Tuple[int, int, int], ...]:
+        """tuple of (int: position, int: rel, int: idx) for each derived-element.
+
+        The first item ("position") refers to position of the derived element's anchor,
+        and the second ("rel") is an integer that can be either -1 and 1 that indicates
+        whether it should be placed before or after its anchor. The idx is the position
+        of the derived element in payload order.
+
+        The `position` int for a subtotal is -1 for anchor "top", sys.maxsize for anchor
+        "bottom", and the position of the anchor for all others.
+
+        Multiple insertions having the same anchor appear in payload order within that
+        group. The strictly increasing insertion index values (1 < 2 < 3) ensure
+        insertions with the same anchor appear in payload order after that anchor.
+        """
+        return tuple(
+            (*self._derived_element_position(element.element_id), idx)
+            for idx, element in enumerate(self._elements)
+            if element.derived
+        )
+
+    def _derived_element_position(self, element_id) -> Tuple[int, int]:
+        """tuple of 2 ints indicating derived element position
+
+        The first item in the return value represents the payload-order base-vector idx
+        which the derived element is "anchor"ed to. The second item can be either a positive 1
+        or negative 1 to indicate whether it should be after or before that element
+        respectively.
+
+        A subtotal with position `(-1, 0)` appears at the top, one with an anchor of
+        `(3, 1)` appears *after* the base row at offset 3; `(sys.maxsize, 0)` is used as
+        the position for a "bottom" anchored subtotal.
+        """
+        anchor = self._elements.get_by_id(element_id).anchor
+        if anchor is None:
+            return (sys.maxsize, 0)
+
+        # --- "top" and "bottom" have fixed position mappings ---
+        if anchor == "top":
+            return (-1, 0)
+        if anchor == "bottom":
+            return (sys.maxsize, 0)
+
+        # --- otherwise the anchor is a dictionary, with an "alias" (matches the
+        # --- element id) and a "position", which can be either "before" or "after"
+        anchor_id = anchor.get("alias")
+        relative_pos = -1 if anchor.get("position") == "before" else 1
+        return (
+            (self._element_positions_by_id[anchor_id], relative_pos)
+            if anchor_id in self._element_positions_by_id
+            else (sys.maxsize, 0)
+        )
+
+    @lazyproperty
+    def _element_order_descriptors(self) -> Tuple[Tuple[int, int, int], ...]:
         """tuple of (position, idx, element_id) triple for each element in dimension.
 
         The `position` of an element is it's index in the ordered collection. The
@@ -225,7 +300,7 @@ class ExplicitOrderCollator(_BaseAnchoredCollator):
           remaining elements appear after, in payload order.
         """
 
-        def iter_element_order_descriptors():
+        def iter_element_order_descriptors() -> Iterator[Tuple[int, int]]:
             """Generate (idx, element_id) pair for each base-vector value.
 
             The (idx, id) pairs are generated in position order. The position of an
@@ -234,8 +309,13 @@ class ExplicitOrderCollator(_BaseAnchoredCollator):
             # --- OrderedDict mapping element-id to payload-order, like {15:0, 12:1,..}.
             # --- This gives us payload-idx lookup along with duplicate and leftover
             # --- tracking.
+            # --- Include derived element ids when enumerating, but not in the final
+            # --- collection because their position must be recalculated for explicit
+            # --- order.
             remaining_element_idxs_by_id = collections.OrderedDict(
-                (id_, idx) for idx, id_ in enumerate(self._element_ids)
+                (element.element_id, idx)
+                for idx, element in enumerate(self._elements)
+                if not element.derived
             )
 
             # --- yield (idx, id) pair for each element mentioned by id in transform,
@@ -270,7 +350,7 @@ class PayloadOrderCollator(_BaseAnchoredCollator):
     """
 
     @lazyproperty
-    def _element_order_descriptors(self):
+    def _element_order_descriptors(self) -> Tuple[Tuple[int, int, int], ...]:
         """tuple of (position, idx, element_id) triple for each element in dimension.
 
         In payload-order, the position of an element is simply it's index in the
@@ -306,7 +386,9 @@ class SortByValueCollator(_BaseCollator):
         self._subtotal_values = subtotal_values
 
     @classmethod
-    def display_order(cls, dimension, element_values, subtotal_values, empty_idxs):
+    def display_order(
+        cls, dimension, element_values, subtotal_values, empty_idxs
+    ) -> Tuple[int, ...]:
         """Return sequence of int element-idxs ordered by sort on `element_values`.
 
         The returned tuple contains a signed-int value for each vector and subtotal of
@@ -317,7 +399,7 @@ class SortByValueCollator(_BaseCollator):
         )._display_order
 
     @property
-    def _display_order(self):
+    def _display_order(self) -> Tuple[int, ...]:
         """tuple of int element-idx specifying ordering of dimension elements.
 
         The element-indices are signed; positive indices are base-elements and negative
@@ -348,7 +430,7 @@ class SortByValueCollator(_BaseCollator):
         )
 
     @lazyproperty
-    def _body_idxs(self):
+    def _body_idxs(self) -> Tuple[int, ...]:
         """tuple of int element-idx for each non-anchored dimension element.
 
         These values appear in sorted order. The sequence is determined by the
@@ -358,7 +440,8 @@ class SortByValueCollator(_BaseCollator):
         If values are nan, they are placed at the end in their original payload order.
         """
         fixed_idxs = frozenset(self._top_fixed_idxs + self._bottom_fixed_idxs)
-        keys, nans = [], []
+        keys: List[Tuple[int, int]] = []
+        nans: List[Tuple[int, int]] = []
         for i, val in enumerate(self._element_values):
             if i not in fixed_idxs:
                 group = nans if self._is_nan(val) else keys
@@ -367,7 +450,7 @@ class SortByValueCollator(_BaseCollator):
         return tuple(idx for _, idx in (sorted(keys, reverse=self._descending) + nans))
 
     @lazyproperty
-    def _bottom_fixed_idxs(self):
+    def _bottom_fixed_idxs(self) -> Tuple[int, ...]:
         """Tuple of (positive) idx of each fixed base element anchored to bottom.
 
         The items appear in the order specified in the "bottom" fix-grouping of the
@@ -376,7 +459,7 @@ class SortByValueCollator(_BaseCollator):
         return tuple(self._iter_fixed_idxs(self._order_spec.bottom_fixed_ids))
 
     @lazyproperty
-    def _bottom_subtotal_idxs(self):
+    def _bottom_subtotal_idxs(self) -> Tuple[int, ...]:
         """Tuple of negative idx of each subtotal vector in order it appears on bottom.
 
         Subtotal vectors all appear as a sorted group at the top of the table when the
@@ -388,7 +471,7 @@ class SortByValueCollator(_BaseCollator):
         return () if self._descending else self._subtotal_idxs
 
     @lazyproperty
-    def _descending(self):
+    def _descending(self) -> bool:
         """True if collation direction is larger-to-smaller, False otherwise.
 
         Descending is the default direction because it is so much more common than
@@ -397,7 +480,7 @@ class SortByValueCollator(_BaseCollator):
         return self._order_spec.descending
 
     @staticmethod
-    def _is_nan(value):
+    def _is_nan(value) -> bool:
         """True if `value` is NaN, False otherwise.
 
         We can't use `np.isnan` directly because we want to treat any string value as
@@ -409,7 +492,7 @@ class SortByValueCollator(_BaseCollator):
         except TypeError:
             return False
 
-    def _iter_fixed_idxs(self, fixed_element_ids):
+    def _iter_fixed_idxs(self, fixed_element_ids) -> Iterator[int]:
         """Generate the element-idx of each element represented by `fixed_element_ids`.
 
         Any element-id specified in the `fixed_element_ids` that is not present in the
@@ -423,7 +506,7 @@ class SortByValueCollator(_BaseCollator):
             yield element_idxs_by_id[fixed_element_id]
 
     @lazyproperty
-    def _subtotal_idxs(self):
+    def _subtotal_idxs(self) -> Tuple[int, ...]:
         """tuple of int (negative) element-idx for each subtotal of this dimension.
 
         These values appear in sorted order. The values are determined by the
@@ -437,7 +520,8 @@ class SortByValueCollator(_BaseCollator):
         n_values = len(subtotal_values)
         # --- `keys` looks like [(75.36, -3), (18.17, -2), (23.46, -1)], providing a
         # --- sequence that can be sorted and then harvested for ordered idxs.
-        keys, nans = [], []
+        keys: List[Tuple[int, int]] = []
+        nans: List[Tuple[int, int]] = []
         for i, val in enumerate(subtotal_values):
             neg_idx = i - n_values
             group = nans if self._is_nan(val) else keys
@@ -446,7 +530,7 @@ class SortByValueCollator(_BaseCollator):
         return tuple(idx for _, idx in (sorted(keys, reverse=self._descending) + nans))
 
     @lazyproperty
-    def _top_fixed_idxs(self):
+    def _top_fixed_idxs(self) -> Tuple[int, ...]:
         """Tuple of (positive) payload-order idx for each top-anchored element.
 
         The items appear in the order specified in the "top" fix-grouping of the
@@ -455,7 +539,7 @@ class SortByValueCollator(_BaseCollator):
         return tuple(self._iter_fixed_idxs(self._order_spec.top_fixed_ids))
 
     @lazyproperty
-    def _top_subtotal_idxs(self):
+    def _top_subtotal_idxs(self) -> Tuple[int, ...]:
         """Tuple of negative idx of each subtotal vector in the order it appears on top.
 
         Subtotal vectors all appear as a sorted group at the top of the table when the
