@@ -19,11 +19,10 @@ import math
 import numpy as np
 from tabulate import tabulate
 
-from cr.cube.enums import CUBE_MEASURE as CM
+from cr.cube.enums import CUBE_MEASURE as CM, DIMENSION_TYPE as DT
 from cr.cube.min_base_size_mask import MinBaseSizeMask
 from cr.cube.matrix import Assembler
 from cr.cube.measures.pairwise_significance import PairwiseSignificance
-from cr.cube.noa.smoothing import SingleSidedMovingAvgSmoother
 from cr.cube.scalar import MeansScalar
 from cr.cube.stripe.assembler import StripeAssembler
 from cr.cube.util import lazyproperty
@@ -79,25 +78,6 @@ class CubePartition:
         """
         return tuple(d.dimension_type for d in self._dimensions)
 
-    def evaluate(self, measure_expr):
-        """Return 1D/2D ndarray result of evaluating `measure_expr`.
-
-        `measure_expr` contains the function to apply and its parameters, like::
-
-            {
-                 "function": "one_sided_moving_avg",
-                 "base_measure": "col_percent",
-                 "window": 3
-            }
-
-        This expression specifies application of the `one_sided_moving_avg` function to
-        the `col_percent` base-measure, with a sliding window of 3 periods.
-        """
-        function = measure_expr.get("function", None)
-        if function != "one_sided_moving_avg":
-            raise NotImplementedError("Function {} is not available.".format(function))
-        return SingleSidedMovingAvgSmoother(self, measure_expr).values
-
     @lazyproperty
     def ndim(self):
         """int count of dimensions for this partition."""
@@ -288,6 +268,16 @@ class _Slice(CubePartition):
 
     # ---interface ---------------------------------------------------
 
+    @lazyproperty
+    def column_aliases(self):
+        """1D str ndarray of alias for each column, for use as column headings."""
+        return self._assembler.column_aliases
+
+    @lazyproperty
+    def column_codes(self):
+        """1D int ndarray of code for each column, for use as column headings."""
+        return self._assembler.column_codes
+
     @lazyproperty
     def column_index(self):
         """2D np.float64 ndarray of column-index "percentage".
@@ -808,6 +798,16 @@ class _Slice(CubePartition):
         """
         return np.stack([self.pvals, self.zscores])
 
+    @lazyproperty
+    def row_aliases(self):
+        """1D str ndarray of alias for each row, for use as row headings."""
+        return self._assembler.row_aliases
+
+    @lazyproperty
+    def row_codes(self):
+        """1D int ndarray of code for each row, for use as row headings."""
+        return self._assembler.row_codes
+
     @lazyproperty
     def row_labels(self):
         """1D str ndarray of name for each row, suitable for use as row headings."""
@@ -887,6 +887,11 @@ class _Slice(CubePartition):
         """
         return self._assembler.rows_base
 
+    @lazyproperty
+    def rows_dimension_alias(self):
+        """str alias assigned to rows-dimension."""
+        return self._rows_dimension.alias
+
     @lazyproperty
     def rows_dimension_description(self):
         """str description assigned to rows-dimension.
@@ -1059,11 +1064,59 @@ class _Slice(CubePartition):
         return self.counts.shape
 
     @lazyproperty
-    def smoothed_dimension_dict(self):
-        """dict, smoothed column dimension definition"""
-        # TODO:  remove this property when the smoother gets the base measure directly
-        # from the matrix later on.
-        return self._columns_dimension._dimension_dict
+    def smoothed_column_index(self):
+        """2D np.float64 ndarray of smoothed column-index "percentage".
+
+        If cube has smoothing specification in the transforms it will return the
+        column index smoothed according to the algorithm and the parameters
+        specified, otherwise it fallbacks to unsmoothed values.
+        """
+        return self._assembler.smoothed_column_index
+
+    @lazyproperty
+    def smoothed_column_percentages(self):
+        """2D np.float64 ndarray of smoothed column-percentages for each matrix cell.
+
+        If cube has smoothing specification in the transforms it will return the
+        column percentages smoothed according to the algorithm and the parameters
+        specified, otherwise it fallbacks to unsmoothed values.
+        """
+        return self.smoothed_column_proportions * 100
+
+    @lazyproperty
+    def smoothed_column_proportions(self):
+        """2D np.float64 ndarray of smoothed column-proportion for each matrix cell.
+
+        If cube has smoothing specification in the transforms it will return the
+        column proportions smoothed according to the algorithm and the parameters
+        specified, otherwise it fallbacks to unsmoothed values.
+        """
+        return self._assembler.smoothed_column_proportions
+
+    @lazyproperty
+    def smoothed_columns_scale_mean(self):
+        """Optional 1D np.float64 ndarray of smoothed scale mean for each column.
+
+        If cube has smoothing specification in the transforms it will return the
+        column scale mean smoothed according to the algorithm and the parameters
+        specified, otherwise it fallbacks to unsmoothed values.
+        """
+        return self._assembler.smoothed_columns_scale_mean
+
+    @lazyproperty
+    def smoothed_means(self):
+        """2D optional np.float64 ndarray of smoothed mean value for each table cell.
+
+        If cube has smoothing specification in the transforms it will return the
+        smoothed means according to the algorithm and the parameters specified,
+        otherwise it fallbacks to unsmoothed values.
+        """
+        try:
+            return self._assembler.smoothed_means
+        except ValueError:
+            raise ValueError(
+                "`.means` is undefined for a cube-result without a mean measure"
+            )
 
     @lazyproperty
     def stddev(self):
@@ -1097,6 +1150,26 @@ class _Slice(CubePartition):
             self, self._alpha, self._only_larger
         ).summary_pairwise_indices
 
+    @lazyproperty
+    def tab_label(self):
+        """Subvar label of slice id if first dimension is a CA_SUBVAR, '"' otherwise."""
+        first_dimension = self._cube.dimensions[0]
+        return (
+            first_dimension.valid_elements[self._slice_idx].label
+            if first_dimension.dimension_type == DT.CA_SUBVAR
+            else ""
+        )
+
+    @lazyproperty
+    def tab_alias(self):
+        """Subvar alias of slice id if first dimension is a CA_SUBVAR, '"' otherwise."""
+        first_dimension = self._cube.dimensions[0]
+        return (
+            first_dimension.valid_elements[self._slice_idx].alias
+            if first_dimension.dimension_type == DT.CA_SUBVAR
+            else ""
+        )
+
     @lazyproperty
     def table_base(self):
         """Scalar or 1D/2D np.float64 ndarray of unweighted-N for table.
@@ -1461,6 +1534,16 @@ class _Strand(CubePartition):
         """
         return self._assembler.row_count
 
+    @lazyproperty
+    def row_aliases(self):
+        """1D str ndarray of alias for each row, for use as row headings."""
+        return self._assembler.row_aliases
+
+    @lazyproperty
+    def row_codes(self):
+        """1D int ndarray of code for each row, for use as row headings."""
+        return self._assembler.row_codes
+
     @lazyproperty
     def row_labels(self):
         """1D str ndarray of name for each row, suitable for use as row headings."""
@@ -1474,6 +1557,11 @@ class _Strand(CubePartition):
         # --- strand.
         return self.unweighted_counts
 
+    @lazyproperty
+    def rows_dimension_alias(self):
+        """str alias assigned to rows-dimension."""
+        return self._rows_dimension.alias
+
     @lazyproperty
     def rows_dimension_fills(self):
         """tuple of optional RGB str like "#def032" fill color for each strand row.
@@ -1557,11 +1645,6 @@ class _Strand(CubePartition):
         """
         return (self.row_count,)
 
-    @lazyproperty
-    def smoothed_dimension_dict(self):
-        """dict, row dimension definition"""
-        return self._rows_dimension._dimension_dict
-
     @lazyproperty
     def share_sum(self):
         """1D np.float64 ndarray of share of sum for each row of strand.
@@ -1578,6 +1661,21 @@ class _Strand(CubePartition):
                 "`.share_sum` is undefined for a cube-result without a sum measure"
             )
 
+    @lazyproperty
+    def smoothed_means(self):
+        """1D np.float64 ndarray of smoothed mean for each row of strand.
+
+        If cube has smoothing specification in the transforms it will return the
+        smoothed means according to the algorithm and the parameters specified,
+        otherwise it fallbacks to unsmoothed values.
+        """
+        try:
+            return self._assembler.smoothed_means
+        except ValueError:
+            raise ValueError(
+                "`.means` is undefined for a cube-result without a mean measure"
+            )
+
     @lazyproperty
     def stddev(self):
         """1D np.float64 ndarray of stddev for each row of strand.
