@@ -55,6 +55,26 @@ class Assembler:
         self._dimensions = dimensions
         self._slice_idx = slice_idx
 
+    @lazyproperty
+    def column_aliases(self):
+        """1D str ndarray of column alias for each matrix row.
+
+        These are suitable for use as column headings; alias for subtotal columns appear
+        in the sequence and alias are ordered to correspond with their respective data
+        column.
+        """
+        return self._dimension_aliases(self._columns_dimension, self._column_order)
+
+    @lazyproperty
+    def column_codes(self):
+        """1D int ndarray of column codes for each matrix row.
+
+        These are suitable for use as column headings; codes for subtotal columns appear
+        in the sequence and codes are ordered to correspond with their respective data
+        column.
+        """
+        return self._dimension_codes(self._columns_dimension, self._column_order)
+
     @lazyproperty
     def column_comparable_counts(self):
         """2D np.float64 ndarray of counts comparable in the column dimension.
@@ -381,6 +401,26 @@ class Assembler:
         """
         return self._assemble_matrix(self._measures.row_comparable_counts.blocks)
 
+    @lazyproperty
+    def row_aliases(self):
+        """1D str ndarray of row alias for each matrix row.
+
+        These are suitable for use as row headings; alias for subtotal rows appear in
+        the sequence and alias are ordered to correspond with their respective data
+        row.
+        """
+        return self._dimension_aliases(self._rows_dimension, self._row_order)
+
+    @lazyproperty
+    def row_codes(self):
+        """1D int ndarray of row codes for each matrix row.
+
+        These are suitable for use as row headings; codes for subtotal rows appear in
+        the sequence and codes are ordered to correspond with their respective data
+        row.
+        """
+        return self._dimension_codes(self._rows_dimension, self._row_order)
+
     @lazyproperty
     def row_labels(self):
         """1D str ndarray of row name for each matrix row.
@@ -518,6 +558,35 @@ class Assembler:
         """Optional 1D Row scale median marginal for this cube-result"""
         return self._assemble_marginal(self._measures.rows_scale_median)
 
+    @lazyproperty
+    def smoothed_column_index(self):
+        """2D np.float64 ndarray of smoothed column-index for each table cell."""
+        return self._assemble_matrix(self._measures.smoothed_column_index.blocks)
+
+    @lazyproperty
+    def smoothed_column_proportions(self):
+        """2D np.float64 ndarray of smoothed column-proportion for each matrix cell.
+
+        This is the proportion of the weighted-count for cell to the weighted-N of the
+        column the cell appears in (aka. column-margin). Generally a number between 0.0
+        and 1.0 inclusive, but subtotal differences can be between -1.0 and 1.0
+        inclusive.
+        """
+        return self._assemble_matrix(self._measures.smoothed_column_proportions.blocks)
+
+    @lazyproperty
+    def smoothed_columns_scale_mean(self):
+        """Optional 1D smoothed column scale mean marginal for this cube-result"""
+        return self._assemble_marginal(self._measures.smoothed_columns_scale_mean)
+
+    @lazyproperty
+    def smoothed_means(self):
+        """2D optional np.float64 ndarray of smoothed mean for each cell.
+
+        Raises `ValueError` if the cube-result does not include a means cube-measure.
+        """
+        return self._assemble_matrix(self._measures.smoothed_means.blocks)
+
     @lazyproperty
     def sums(self):
         """2D optional np.float64 ndarray of sum for each cell.
@@ -752,6 +821,24 @@ class Assembler:
             if derived
         )
 
+    def _dimension_aliases(self, dimension, order):
+        """1D str ndarray of alias for each vector of `dimension`.
+
+        These are suitable for use as row/column headings; alias for subtotals appear
+        in the sequence specified by `order`, such that aliases are ordered to
+        correspond with their respective data vector.
+        """
+        return np.array(dimension.element_aliases + dimension.subtotal_aliases)[order]
+
+    def _dimension_codes(self, dimension, order):
+        """1D str ndarray of codes for each vector of `dimension`.
+
+        These are suitable for use as row/column headings; codes for subtotals appear
+        in the sequence specified by `order`, such that codes are ordered to
+        correspond with their respective data vector.
+        """
+        return np.array(dimension.element_ids + dimension.insertion_ids)[order]
+
     def _dimension_labels(self, dimension, order):
         """1D str ndarray of name for each vector of `dimension`.
 
