@@ -110,6 +110,30 @@ class StripeAssembler:
         """
         return len(self._row_order)
 
+    @lazyproperty
+    def row_aliases(self):
+        """1D str ndarray of row alias for each matrix row.
+
+        These are suitable for use as row headings; aliases for subtotal rows appear in
+        the sequence and aliases are ordered to correspond with their respective data
+        row.
+        """
+        return np.array(
+            self._rows_dimension.element_aliases + self._rows_dimension.subtotal_aliases
+        )[self._row_order]
+
+    @lazyproperty
+    def row_codes(self):
+        """1D int ndarray of row code for each matrix row.
+
+        These are suitable for use as row headings; codes for subtotal rows appear in
+        the sequence and codes are ordered to correspond with their respective data
+        row.
+        """
+        return np.array(
+            self._rows_dimension.element_ids + self._rows_dimension.insertion_ids
+        )[self._row_order]
+
     @lazyproperty
     def row_labels(self):
         """1D str ndarray of row name for each matrix row.
@@ -188,6 +212,14 @@ class StripeAssembler:
         """
         return self._assemble_vector(self._measures.share_sum.blocks)
 
+    @lazyproperty
+    def smoothed_means(self):
+        """1D np.float64 ndarray of smoothed mean for each row.
+
+        Raises ValueError when the cube-result does not include a means cube-measure.
+        """
+        return self._assemble_vector(self._measures.smoothed_means.blocks)
+
     @lazyproperty
     def stddev(self):
         """1D np.float64 ndarray of stddev for each row.
