@@ -14,7 +14,7 @@ from cr.cube.matrix.cubemeasure import _BaseCubeOverlaps
 from ..fixtures import CR, OL
 
 
-class DescribeAssembler(object):
+class DescribeAssembler:
     """Integration-test suite for `cr.cube.matrix.Assembler`."""
 
     def it_computes_column_proportions_for_cat_x_cat(self):
@@ -62,16 +62,20 @@ class DescribeAssembler(object):
 
     def it_computes_column_unweighted_bases_for_cat_hs_x_mr(self):
         slice_ = Cube(CR.CAT_HS_X_MR).partitions[0]
-        assert slice_.column_unweighted_bases.tolist() == [
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-            [26, 76, 118, 369, 385],
-        ]
+        assert slice_.column_unweighted_bases == pytest.approx(
+            np.array(
+                [
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                    [26, 76, 118, 369, 385],
+                ]
+            )
+        )
 
     def it_computes_column_unweighted_bases_for_mr_x_cat_hs(self):
         slice_ = Cube(CR.MR_X_CAT_HS_MT).partitions[0]
@@ -1503,7 +1507,7 @@ class DescribeAssembler(object):
         )
 
 
-class DescribeCubemeasure(object):
+class DescribeCubemeasure:
     def it_provides_overlaps_for_cat_x_mr_sub_x_mr_sel(self):
         cube_dict = OL.CAT_X_MR_SUB_X_MR_SEL
         overlaps_measure = _BaseCubeOverlaps.factory(
@@ -1656,7 +1660,7 @@ class DescribeCubemeasure(object):
         ]
 
 
-class Describe_BaseOrderHelper(object):
+class Describe_BaseOrderHelper:
     """Integration-test suite for `cr.cube.matrix._BaseOrderHelper`."""
 
     @pytest.mark.parametrize(
@@ -1714,7 +1718,7 @@ class Describe_BaseOrderHelper(object):
         assert column_display_order.tolist() == expected_value
 
 
-class Describe_SortRowsByColumnValueHelper(object):
+class Describe_SortRowsByColumnValueHelper:
     """Integration-test suite for `cr.cube.matrix._SortRowsByColumnValueHelper`."""
 
     @pytest.mark.parametrize(
@@ -1785,7 +1789,7 @@ class Describe_SortRowsByColumnValueHelper(object):
         assert assembler._row_order.tolist() == expected_value
 
 
-class Describe_SortRowsByInsertedColumnHelper(object):
+class Describe_SortRowsByInsertedColumnHelper:
     """Integration-test suite for `cr.cube.matrix._SortRowsByInsertedColumnHelper`."""
 
     @pytest.mark.parametrize(
@@ -1800,7 +1804,7 @@ class Describe_SortRowsByInsertedColumnHelper(object):
             ),
             (
                 CR.CA_SUBVAR_X_CA_CAT_HS,
-                "col_percent",
+                "table_percent",
                 "ascending",
                 {},
                 [2, 1, 0],
@@ -1824,3 +1828,18 @@ class Describe_SortRowsByInsertedColumnHelper(object):
         assembler = _Slice(Cube(fixture), 0, transforms, None, 0)._assembler
 
         assert assembler._row_order.tolist() == expected_value
+
+    def but_it_fallback_to_payload_order_when_sort_by_value_is_not_supported(self):
+        transforms = {
+            "rows_dimension": {
+                "order": {
+                    "type": "opposing_insertion",
+                    "insertion_id": 1,
+                    "measure": "population_counts",
+                    "direction": "ascending",
+                }
+            }
+        }
+        assembler = _Slice(Cube(CR.CAT_HS_X_CAT_HS), 0, transforms, None, 0)._assembler
+
+        assert assembler._row_order.tolist() == [0, 1, -2, 2, 3, 4, -1]
