@@ -1,20 +1,26 @@
-# encoding: utf-8 """Unit test suite for `cr.cube.stripe.assembler` module."""
+# encoding: utf-8
+
+"""Unit test suite for `cr.cube.stripe.assembler` module."""
 
 import numpy as np
 import pytest
 
 from cr.cube.cube import Cube
-from cr.cube.dimension import Dimension, _Element, _OrderSpec, _Subtotal
+from cr.cube.dimension import Dimension, _Element, _OrderSpec
 from cr.cube.enums import COLLATION_METHOD as CM
 from cr.cube.stripe.assembler import (
     _BaseOrderHelper,
+    _BaseSortByValueHelper,
     _OrderHelper,
+    _SortByLabelHelper,
     _SortByMeasureHelper,
     StripeAssembler,
 )
 from cr.cube.stripe.measure import (
     _BaseSecondOrderMeasure,
     _Means,
+    _PopulationProportions,
+    _PopulationProportionStderrs,
     _ScaledCounts,
     StripeMeasures,
     _TableProportionStddevs,
@@ -34,13 +40,15 @@ from ...unitutil import (
 )
 
 
-class DescribeStripeAssembler(object):
+class DescribeStripeAssembler:
     """Unit test suite for `cr.cube.stripe.assembler.StripeAssembler` object."""
 
     @pytest.mark.parametrize(
         "measure_prop_name, MeasureCls",
         (
             ("means", _Means),
+            ("population_proportions", _PopulationProportions),
+            ("population_proportion_stderrs", _PopulationProportionStderrs),
             ("table_proportion_stddevs", _TableProportionStddevs),
             ("table_proportion_stderrs", _TableProportionStderrs),
             ("table_proportions", _TableProportions),
@@ -85,14 +93,9 @@ class DescribeStripeAssembler(object):
 
         assert assembler.row_count == 5
 
-    def it_knows_the_row_labels(self, request, rows_dimension_, _row_order_prop_):
-        rows_dimension_.valid_elements = tuple(
-            instance_mock(request, _Element, label=label)
-            for label in ("baz", "foo", "bar")
-        )
-        rows_dimension_.subtotals = tuple(
-            instance_mock(request, _Subtotal, label=label) for label in ("bing", "bada")
-        )
+    def it_knows_the_row_labels(self, rows_dimension_, _row_order_prop_):
+        rows_dimension_.element_labels = ("baz", "foo", "bar")
+        rows_dimension_.subtotal_labels = ("bing", "bada")
         _row_order_prop_.return_value = np.array([1, 2, 0, -1, -2])
         assembler = StripeAssembler(None, rows_dimension_, None, None)
 
@@ -241,13 +244,14 @@ class DescribeStripeAssembler(object):
         return instance_mock(request, _ScaledCounts)
 
 
-class Describe_BaseOrderHelper(object):
+class Describe_BaseOrderHelper:
     """Unit-test suite for `cr.cube.stripe.assembler._BaseOrderHelper` object."""
 
     @pytest.mark.parametrize(
         "collation_method, HelperCls",
         (
             (CM.UNIVARIATE_MEASURE, _SortByMeasureHelper),
+            (CM.LABEL, _SortByLabelHelper),
             (CM.EXPLICIT_ORDER, _OrderHelper),
             (CM.PAYLOAD_ORDER, _OrderHelper),
         ),
@@ -295,7 +299,7 @@ class Describe_BaseOrderHelper(object):
         return instance_mock(request, StripeMeasures)
 
 
-class Describe_OrderHelper(object):
+class Describe_OrderHelper:
     """Unit test suite for `cr.cube.stripe.assembler._OrderHelper` object."""
 
     @pytest.mark.parametrize(
@@ -328,30 +332,24 @@ class Describe_OrderHelper(object):
         assert display_order == (1, -2, 3, 5, -1)
 
 
-class Describe_SortByMeasureHelper(object):
-    """Unit test suite for `cr.cube.strip.assembler._SortByMeasureHelper`."""
+class Describe_BaseSortByValueHelper:
+    """Unit test suite for `cr.cube.strip.assembler._BaseSortByValueHelper`."""
 
-    def it_computes_the_display_order_to_help(self, request, dimension_):
-        property_mock(
-            request,
-            _SortByMeasureHelper,
-            "_element_values",
-            # --- return type is ndarray in real life, but assert_called_once_with()
-            # --- won't match on those, so use list instead.
-            return_value=[16, 3, 12],
-        )
-        property_mock(
-            request,
-            _SortByMeasureHelper,
-            "_subtotal_values",
-            return_value=[15, 19],  # --- ndarray in real life ---
-        )
-        property_mock(request, _SortByMeasureHelper, "_empty_row_idxs", return_value=())
-        SortByValueCollator_ = class_mock(
-            request, "cr.cube.stripe.assembler.SortByValueCollator"
-        )
+    def it_computes_the_display_order_to_help(
+        self,
+        dimension_,
+        _element_values_prop_,
+        _subtotal_values_prop_,
+        _empty_row_idxs_prop_,
+        SortByValueCollator_,
+    ):
+        # --- return type of first two is ndarray in real life, but
+        # --- assert_called_once_with() won't match on those, so use list instead.
+        _element_values_prop_.return_value = [16, 3, 12]
+        _subtotal_values_prop_.return_value = [15, 19]
+        _empty_row_idxs_prop_.return_value = ()
         SortByValueCollator_.display_order.return_value = (-1, -2, 0, 2, 1)
-        order_helper = _SortByMeasureHelper(dimension_, None)
+        order_helper = _BaseSortByValueHelper(dimension_, None)
 
         order = order_helper._display_order
 
@@ -360,6 +358,78 @@ class Describe_SortByMeasureHelper(object):
         )
         assert order == (-1, -2, 0, 2, 1)
 
+    def but_it_falls_back_to_payload_order_on_value_error(
+        self,
+        request,
+        dimension_,
+        _element_values_prop_,
+        _subtotal_values_prop_,
+        _empty_row_idxs_prop_,
+        SortByValueCollator_,
+    ):
+        _element_values_prop_.return_value = None
+        _subtotal_values_prop_.return_value = None
+        _empty_row_idxs_prop_.return_value = (4, 2)
+        SortByValueCollator_.display_order.side_effect = ValueError
+        PayloadOrderCollator_ = class_mock(
+            request, "cr.cube.stripe.assembler.PayloadOrderCollator"
+        )
+        PayloadOrderCollator_.display_order.return_value = (1, 2, 3, 4)
+        order_helper = _BaseSortByValueHelper(dimension_, None)
+
+        order = order_helper._display_order
+
+        PayloadOrderCollator_.display_order.assert_called_once_with(dimension_, (4, 2))
+        assert order == (1, 2, 3, 4)
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def dimension_(self, request):
+        return instance_mock(request, Dimension)
+
+    @pytest.fixture
+    def _element_values_prop_(self, request):
+        return property_mock(request, _BaseSortByValueHelper, "_element_values")
+
+    @pytest.fixture
+    def _empty_row_idxs_prop_(self, request):
+        return property_mock(request, _BaseSortByValueHelper, "_empty_row_idxs")
+
+    @pytest.fixture
+    def SortByValueCollator_(self, request):
+        return class_mock(request, "cr.cube.stripe.assembler.SortByValueCollator")
+
+    @pytest.fixture
+    def _subtotal_values_prop_(self, request):
+        return property_mock(request, _BaseSortByValueHelper, "_subtotal_values")
+
+
+class Describe_SortByLabelHelper:
+    """Unit test suite for `cr.cube.strip.assembler._SortByLabelHelper`."""
+
+    def it_extracts_the_element_values_to_help(self, dimension_):
+        dimension_.element_labels = ["b", "a", "c"]
+        order_helper = _SortByLabelHelper(dimension_, None)
+
+        assert order_helper._element_values.tolist() == ["b", "a", "c"]
+
+    def it_extracts_the_subtotal_values_to_help(self, dimension_):
+        dimension_.subtotal_labels = ["b", "a", "c"]
+        order_helper = _SortByLabelHelper(dimension_, None)
+
+        assert order_helper._subtotal_values.tolist() == ["b", "a", "c"]
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def dimension_(self, request):
+        return instance_mock(request, Dimension)
+
+
+class Describe_SortByMeasureHelper:
+    """Unit test suite for `cr.cube.strip.assembler._SortByMeasureHelper`."""
+
     def it_extracts_the_element_values_to_help(self, _measure_prop_, measure_):
         _measure_prop_.return_value = measure_
         measure_.blocks = [np.arange(5), None]
@@ -367,18 +437,41 @@ class Describe_SortByMeasureHelper(object):
 
         assert order_helper._element_values.tolist() == [0, 1, 2, 3, 4]
 
+    @pytest.mark.parametrize(
+        "json_name, internal_name",
+        (
+            ("base_unweighted", "unweighted_bases"),
+            ("base_weighted", "weighted_bases"),
+            ("count_unweighted", "unweighted_counts"),
+            ("count_weighted", "weighted_counts"),
+            ("mean", "means"),
+            ("percent", "table_proportions"),
+            ("percent_stddev", "table_proportion_stddevs"),
+            ("percent_moe", "table_proportion_stderrs"),
+            ("population", "population_proportions"),
+            ("population_moe", "population_proportion_stderrs"),
+            ("sum", "sums"),
+        ),
+    )
     def it_retrieves_the_right_measure_object_to_help(
-        self, request, _order_spec_prop_, order_spec_, measure_
+        self,
+        request,
+        _order_spec_prop_,
+        order_spec_,
+        measure_,
+        json_name,
+        internal_name,
     ):
-        measures_ = instance_mock(request, StripeMeasures, unweighted_bases=measure_)
+        measures_ = instance_mock(request, StripeMeasures)
+        setattr(measures_, internal_name, measure_)
         _order_spec_prop_.return_value = order_spec_
-        order_spec_.measure_keyname = "base_unweighted"
+        order_spec_.measure_keyname = json_name
         order_helper = _SortByMeasureHelper(None, measures_)
 
         assert order_helper._measure is measure_
 
     def but_it_raises_when_the_sort_measure_is_not_supported(
-        self, request, _order_spec_prop_, order_spec_
+        self, _order_spec_prop_, order_spec_
     ):
         _order_spec_prop_.return_value = order_spec_
         order_spec_.measure_keyname = "foobar"
@@ -398,10 +491,6 @@ class Describe_SortByMeasureHelper(object):
 
     # fixture components ---------------------------------------------
 
-    @pytest.fixture
-    def dimension_(self, request):
-        return instance_mock(request, Dimension)
-
     @pytest.fixture
     def measure_(self, request):
         return instance_mock(request, _BaseSecondOrderMeasure)
