@@ -5,7 +5,13 @@
 import pytest
 import numpy as np
 
-from cr.cube.cube import Cube, CubeSet, _Measures, _UnweightedValidCountsMeasure
+from cr.cube.cube import (
+    Cube,
+    CubeSet,
+    _Measures,
+    _BaseMeasure,
+    _UnweightedValidCountsMeasure,
+)
 from cr.cube.cubepart import _Slice, _Strand, _Nub
 from cr.cube.enums import DIMENSION_TYPE as DT
 
@@ -13,7 +19,7 @@ from ..fixtures import CR  # ---mnemonic: CR = 'cube-response'---
 from ..unitutil import call, class_mock, instance_mock, property_mock
 
 
-class DescribeCubeSet(object):
+class DescribeCubeSet:
     """Unit-test suite for `cr.cube.cube.CubeSet` object."""
 
     def it_knows_its_availabe_measures(self, cube_, _cubes_prop_):
@@ -23,6 +29,17 @@ class DescribeCubeSet(object):
 
         assert cube_set.available_measures == {"mean", "sum"}
 
+    def but_it_includes_availabe_measures_from_all_cubes_in_cube_set(
+        self, request, _cubes_prop_
+    ):
+        _cubes_prop_.return_value = tuple(
+            instance_mock(request, Cube, available_measures=measures)
+            for measures in ({"mean"}, {"sum", "counts"})
+        )
+        cube_set = CubeSet(None, None, None, None)
+
+        assert cube_set.available_measures == {"mean", "sum", "counts"}
+
     @pytest.mark.parametrize(
         ("cubes_dimtypes", "expected_value"),
         (
@@ -295,7 +312,7 @@ class DescribeCubeSet(object):
         return property_mock(request, CubeSet, "_is_numeric_measure")
 
 
-class DescribeCube(object):
+class DescribeCube:
     """Unit-test suite for `cr.cube.cube.Cube` object."""
 
     def it_provides_the_default_repr_when_enhanced_repr_fails(
@@ -660,7 +677,7 @@ class DescribeCube(object):
         return property_mock(request, Cube, "dimension_types")
 
 
-class DescribeMeasures(object):
+class DescribeMeasures:
     def it_knows_the_population_fraction(self):
         cube_dict, expected_value = (
             {
@@ -759,7 +776,30 @@ class DescribeMeasures(object):
         assert population_fraction == expected_value
 
 
-class Describe_UweightedValidCountsMeasure(object):
+class Describe_BaseMeasure:
+    def it_returns_None_when_not_able_to_reshape(
+        self, _shape_prop_, _flat_values_prop_
+    ):
+        _shape_prop_.return_value = (4,)
+        _flat_values_prop_.return_value = np.array([1])
+        measure = _BaseMeasure(None, None)
+
+        raw_array = measure.raw_cube_array
+
+        assert raw_array is None
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def _shape_prop_(self, request):
+        return property_mock(request, _BaseMeasure, "_shape")
+
+    @pytest.fixture
+    def _flat_values_prop_(self, request):
+        return property_mock(request, _BaseMeasure, "_flat_values")
+
+
+class Describe_UweightedValidCountsMeasure:
     @pytest.mark.parametrize(
         "valid_counts, expected_value",
         (({"valid_count_unweighted": {"data": [3, 2, 1]}}, [3, 2, 1]), ({}, None)),
